'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = booster;

var _stream = require('stream');

var _nodeObjectHash = require('node-object-hash');

var _nodeObjectHash2 = _interopRequireDefault(_nodeObjectHash);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hashCoerce = (0, _nodeObjectHash2.default)({
    sort: false,
    coerce: true
});

var computeHash = function computeHash(commands, environment, chunk) {
    var commandsHash = hashCoerce.hash(commands);
    var environmentHash = hashCoerce.hash(environment);
    var firstChunkHash = hashCoerce.hash(chunk);
    var hashs = [commandsHash, environmentHash, firstChunkHash];
    (0, _debug2.default)('ezs')('Compute cache hash with', hashs.map(function (h) {
        return h.slice(0, 5).concat('...');
    }));
    return hashCoerce.hash(hashs);
};

/**
 * Takes an `Object` delegate processing to an external pipeline
 *
 * @param {String} [file] the external pipeline is descrbied in a file
 * @param {String} [script] the external pipeline is descrbied in a sting of characters
 * @param {String} [commands] the external pipeline is descrbied in object
 * @returns {Object}
 */
function booster(data, feed) {
    var _this = this;

    var ezs = this.ezs;

    if (this.isFirst()) {
        var file = this.getParam('file');
        var fileContent = ezs.loadScript(file);
        var script = this.getParam('script', fileContent);
        var cmds = ezs.compileScript(script);
        var commands = this.getParam('commands', cmds.get());
        var environment = this.getEnv();

        if (!commands || commands.length === 0) {
            return feed.stop(new Error('Invalid parmeter for delegate'));
        }

        var streams = ezs.compileCommands(commands, environment);
        this.input = new _stream.PassThrough({ objectMode: true });

        this.whenReady = new Promise(function (getup) {
            var uniqHash = computeHash(commands, environment, data);
            var resetCacheOnError = function resetCacheOnError(error, action) {
                (0, _debug2.default)('ezs')('Error while ' + action + ' cache with hash', uniqHash, error);
                ezs.getCache().del(uniqHash).catch(function (error1) {
                    (0, _debug2.default)('ezs')('Error while deleting cache with hash', uniqHash, error1);
                });
            };

            ezs.getCache().has(uniqHash).then(function (cached) {
                return new Promise(function (resolve) {
                    _this.cached = cached;
                    if (cached) {
                        (0, _debug2.default)('ezs')('Using cache with hash', uniqHash);
                        console.log(uniqHash);
                        _this.emit('cache:connected', uniqHash);
                        return ezs.getCache().get(uniqHash).catch(function (err) {
                            return feed.stop(err);
                        }).then(function (stream) {
                            stream.pipe(ezs.uncompress(ezs.encodingMode())).on('error', function (e1) {
                                resetCacheOnError(e1, 'reading');
                                feed.stop();
                            }).pipe(ezs('unpack')).pipe(ezs('ungroup')).pipe(ezs.catch(function (e) {
                                return e;
                            })).on('error', function (e) {
                                return feed.stop(e);
                            }).on('data', function (d) {
                                return feed.write(d);
                            }).on('end', function () {
                                (0, _debug2.default)('ezs')('Cache with hash', uniqHash, 'was readed');
                                feed.close();
                            });
                        });
                    }
                    (0, _debug2.default)('ezs')('Creating cache with hash', uniqHash);
                    _this.emit('cache:created', uniqHash);
                    _this.input = new _stream.PassThrough(ezs.objectMode());
                    var output = ezs.createPipeline(_this.input, streams).pipe(ezs.catch(function (e) {
                        return e;
                    })).on('error', function (e) {
                        return feed.write(e);
                    }).on('data', function (d) {
                        return feed.write(d);
                    }).pipe(ezs('group')).pipe(ezs('pack')).pipe(ezs.catch(function (e) {
                        return e;
                    })).on('error', function (e) {
                        return resetCacheOnError(e, 'before saving');
                    }).pipe(ezs.compress(ezs.encodingMode())).on('error', function (e) {
                        return resetCacheOnError(e, 'after saving');
                    });

                    _this.whenFinish = new Promise(function (done) {
                        ezs.getCache().set(uniqHash, output).then(function () {
                            (0, _debug2.default)('ezs')('Cache has created with hash', uniqHash);
                            done();
                        }).catch(function (e) {
                            resetCacheOnError(e, 'creating');
                            feed.stop();
                        });
                    });
                    (0, _debug2.default)('ezs')('Delegate first chunk #' + _this.getIndex() + ' containing ' + (Object.keys(data).length || 0) + ' keys');
                    return ezs.writeTo(_this.input, data, function () {
                        feed.end();
                        getup();
                        return resolve();
                    });
                });
            });
        });
        return 1;
    }
    return this.whenReady.then(function () {
        if (_this.isLast()) {
            _this.whenFinish.then(function () {
                return feed.close();
            }).catch(function (e) {
                return feed.stop(e);
            });
            return _this.input.end();
        }
        (0, _debug2.default)('ezs')('Delegate chunk #' + _this.getIndex() + ' containing ' + (Object.keys(data).length || 0) + ' keys');
        return ezs.writeTo(_this.input, data, function () {
            return feed.end();
        });
    }).catch(function (e) {
        return feed.stop(e);
    });
}